import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimAdvancedMsgListener.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimConversationListener.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimFriendshipListener.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimGroupListener.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_conversation.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_friend_application.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_friend_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_change_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_member_change_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_member_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_receipt.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_value_callback.dart';
import 'package:tencent_cloud_chat_sdk/tencent_im_sdk_plugin.dart';

class ListenerConfig with ChangeNotifier, DiagnosticableTreeMixin {
  int _count = 0;

  int get count => _count;

  void increment() {
    _count++;
    notifyListeners();
  }

  String _msgListenerData = '';
  String get msgListenerData => _msgListenerData;

  late V2TimAdvancedMsgListener advancedMsgListener;

  void addAdvancedMsgListener() {
    advancedMsgListener = V2TimAdvancedMsgListener(
      onRecvC2CReadReceipt: (List<V2TimMessageReceipt> receiptList) {
        _msgListenerData = 'onRecvC2CReadReceipt $receiptList';
        notifyListeners();

        //单聊已读回调
      },
      onRecvMessageModified: (V2TimMessage message) {
        _msgListenerData = 'onRecvMessageModified ${message.toJson()}';
        notifyListeners();

        // msg 为被修改之后的消息对象
      },
      onRecvMessageReadReceipts: (List<V2TimMessageReceipt> receiptList) {
        _msgListenerData = 'onRecvMessageReadReceipts $receiptList';
        notifyListeners();
      },
      onRecvMessageRevoked: (String messageid) {
        // 在本地维护的消息中处理被对方撤回的消息
        _msgListenerData = 'onRecvMessageRevoked $messageid';
        notifyListeners();
      },
      onRecvNewMessage: (V2TimMessage message) async {
        if (kDebugMode) {
          print('onRecvNewMessage ${message.toJson()}');
        }

        _msgListenerData = 'onRecvNewMessage ${message.toJson()}';
        notifyListeners();
      },
      onSendMessageProgress: (V2TimMessage message, int progress) {
        //文件上传进度回调

        _msgListenerData = 'onSendMessageProgress ${message.toJson()}';
        notifyListeners();
      },
    );

    TencentImSDKPlugin.v2TIMManager
        .getMessageManager()
        .addAdvancedMsgListener(listener: advancedMsgListener);
  }

  void removeAdvancedMessageListener() {
    TencentImSDKPlugin.v2TIMManager
        .getMessageManager()
        .removeAdvancedMsgListener(listener: advancedMsgListener);
  }

  String _conversationListenerData = '';
  String get conversationListenerData => _conversationListenerData;
  late V2TimConversationListener conversationListener;

  void addConversationListener() {
    conversationListener = V2TimConversationListener(
      onConversationChanged: (List<V2TimConversation> conversationList) {
        //某些会话的关键信息发生变化（未读计数发生变化、最后一条消息被更新等等）的回调函数
        //conversationList    变化的会话列表
        _conversationListenerData =
            'onConversationChanged ${conversationList.toString()}';
        notifyListeners();
      },
      onConversationGroupCreated:
          (String groupName, List<V2TimConversation> conversationList) {
        // 会话分组被创建
        // groupName 会话分组名称
        // conversationList 会话分组包含的会话列表
        _conversationListenerData =
            'onConversationGroupCreated groupName: ${groupName} conversationList ${conversationList.toString()}';
        notifyListeners();
      },
      onConversationGroupDeleted: (String groupName) {
        // 会话分组被删除
        // groupName  被删除的会话分组名称
        _conversationListenerData = 'onConversationGroupDeleted ${groupName}';
        notifyListeners();
      },
      onConversationGroupNameChanged: (String oldName, String newName) {
        // 会话分组名变更
        // oldName 旧名称
        // newName 新名称
        _conversationListenerData =
            'onConversationGroupNameChanged oldName: ${oldName}, newName: ${newName}';
        notifyListeners();
      },
      onConversationsAddedToGroup:
          (String groupName, List<V2TimConversation> conversationList) {
        // 会话分组新增会话
        // groupName 会话分组名称
        // conversationList 被加入会话分组的会话列表
        _conversationListenerData =
            'onConversationsAddedToGroup groupName: ${groupName}, conversationList: ${conversationList.toString()}';
        notifyListeners();
      },
      onConversationsDeletedFromGroup:
          (String groupName, List<V2TimConversation> conversationList) {
        // 会话分组删除会话
        // groupName 会话分组名称
        // conversationList 被删除的会话列表
        _conversationListenerData =
            'onConversationsDeletedFromGroup groupName:${groupName} conversationList: ${conversationList.toString()}';
        notifyListeners();
      },
      onNewConversation: (List<V2TimConversation> conversationList) {
        // 新会话的回调函数
        // conversationList 收到的新会话列表
        _conversationListenerData =
            'onNewConversation ${conversationList.toString()}';
        notifyListeners();
      },
      onSyncServerFailed: () {
        // 同步服务失败的回调函数
        _conversationListenerData = 'onSyncServerFailed';
        notifyListeners();
      },
      onSyncServerFinish: () {
        // 同步服务完成的回调函数
        _conversationListenerData = 'onSyncServerFinish';
        notifyListeners();
      },
      onSyncServerStart: () {
        // 同步服务开始的回调函数
        _conversationListenerData = 'onSyncServerStart';
        notifyListeners();
      },
      onTotalUnreadMessageCountChanged: (int totalUnreadCount) {
        // 会话未读总数改变的回调函数
        // totalUnreadCount 会话未读总数
        _conversationListenerData =
            'onTotalUnreadMessageCountChanged ${totalUnreadCount}';
        notifyListeners();
      },
    );
    TencentImSDKPlugin.v2TIMManager
        .getConversationManager()
        .addConversationListener(listener: conversationListener);
  }

  void removeConversationListener() {
    TencentImSDKPlugin.v2TIMManager
        .getConversationManager()
        .removeConversationListener(listener: conversationListener);
  }

  String _friendshipListenerData = '';
  String get friendshipListenerData => _friendshipListenerData;
  late V2TimFriendshipListener friendshipListener;

  void addFriendshipListener() {
    friendshipListener = V2TimFriendshipListener(
      onBlackListAdd: (List<V2TimFriendInfo> infoList) async {
        //黑名单列表新增用户的回调
        //infoList 新增的用户信息列表
        _friendshipListenerData = 'onBlackListAdd ${infoList.toString()}';
        notifyListeners();
      },
      onBlackListDeleted: (List<String> userList) async {
        //黑名单列表删除的回调
        //userList 被删除的用户id列表
        _friendshipListenerData = 'onBlackListDeleted ${userList.toString()}';
        notifyListeners();
      },
      onFriendApplicationListAdded:
          (List<V2TimFriendApplication> applicationList) async {
        //好友请求数量增加的回调
        //applicationList 新增的好友请求信息列表
        _friendshipListenerData =
            'onFriendApplicationListAdded ${applicationList.toString()}';
        notifyListeners();
      },
      onFriendApplicationListDeleted: (List<String> userIDList) async {
        //好友请求数量减少的回调
        //减少的好友请求的请求用户id列表
        _friendshipListenerData =
            'onFriendApplicationListDeleted ${userIDList.toString()}';
        notifyListeners();
      },
      onFriendApplicationListRead: () async {
        //好友请求已读的回调
        _friendshipListenerData = 'onFriendApplicationListRead';
        notifyListeners();
      },
      onFriendInfoChanged: (List<V2TimFriendInfo> infoList) async {
        //好友信息改变的回调
        //infoList 好友信息改变的好友列表
        _friendshipListenerData = 'onFriendInfoChanged ${infoList.toString()}';
        notifyListeners();
      },
      onFriendListAdded: (List<V2TimFriendInfo> users) async {
        //好友列表增加人员的回调
        //users 新增的好友信息列表
        _friendshipListenerData = 'onFriendListAdded ${users.toString()}';
        notifyListeners();
      },
      onFriendListDeleted: (List<String> userList) async {
        //好友列表减少人员的回调
        //userList 减少的好友id列表
        _friendshipListenerData = 'onFriendListDeleted ${userList.toString()}';
        notifyListeners();
      },
    );
    TencentImSDKPlugin.v2TIMManager
        .getFriendshipManager()
        .addFriendListener(listener: friendshipListener);
  }

  void removeFriendshipListener() {
    TencentImSDKPlugin.v2TIMManager
        .getFriendshipManager()
        .removeFriendListener(listener: friendshipListener);
  }

  String _groupListenerData = '';
  String get groupListenerData => _groupListenerData;
  late V2TimGroupListener groupListener;
  void addGroupListener() {
    groupListener = V2TimGroupListener(
      onApplicationProcessed: (String groupID, V2TimGroupMemberInfo opUser,
          bool isAgreeJoin, String opReason) async {
        _groupListenerData =
            'onApplicationProcessed groupID: ${groupID}, opUser: ${opUser.toJson()}, isAgreeJoin: ${isAgreeJoin}, opReason: ${opReason}';
        notifyListeners();
      },
      onGrantAdministrator: (String groupID, V2TimGroupMemberInfo opUser,
          List<V2TimGroupMemberInfo> memberList) async {
        //指定管理员身份
        //groupID    群 ID
        //opUser    处理人
        //memberList    被处理的群成员
        _groupListenerData =
            'onGrantAdministrator groupID: ${groupID}, opUser: ${opUser}';
        notifyListeners();
      },
      onGroupAttributeChanged:
          (String groupID, Map<String, String> groupAttributeMap) async {
        //收到群属性更新的回调
        //groupID    群 ID
        //groupAttributeMap    群的所有属性
        _groupListenerData = 'onGroupAttributeChanged groupID: ${groupID}';
        notifyListeners();
      },
      onGroupCreated: (String groupID) async {
        //创建群（主要用于多端同步）
        //groupID    群 ID
        _groupListenerData = 'onGroupCreated ${groupID}';
        notifyListeners();
      },
      onGroupDismissed: (String groupID, V2TimGroupMemberInfo opUser) async {
        //群被解散了（全员能收到）
        //groupID    群 ID
        //opUser    处理人
        _groupListenerData = 'onGroupDismissed ${groupID}';
        notifyListeners();
      },
      onGroupInfoChanged:
          (String groupID, List<V2TimGroupChangeInfo> changeInfos) async {
        //群信息被修改（全员能收到）
        //groupID    群 ID
        //changeInfos    修改的群信息
        _groupListenerData = 'onGroupInfoChanged ${groupID}';
        notifyListeners();
      },
      onGroupRecycled: (String groupID, V2TimGroupMemberInfo opUser) async {
        //群被回收（全员能收到）
        //groupID    群 ID
        //opUser    处理人
        _groupListenerData = 'onGroupRecycled ${groupID}';
        notifyListeners();
      },
      onMemberEnter:
          (String groupID, List<V2TimGroupMemberInfo> memberList) async {
        //有用户加入群（全员能够收到）
        //groupID    群 ID
        //memberList    加入的成员
        _groupListenerData = 'onMemberEnter ${groupID}';
        notifyListeners();
      },
      onMemberInfoChanged: (String groupID,
          List<V2TimGroupMemberChangeInfo>
              v2TIMGroupMemberChangeInfoList) async {
        //群成员信息被修改，仅支持禁言通知（全员能收到）。
        //groupID    群 ID
        //v2TIMGroupMemberChangeInfoList    被修改的群成员信息
        _groupListenerData = 'onMemberInfoChanged ${groupID}';
        notifyListeners();
      },
      onMemberInvited: (String groupID, V2TimGroupMemberInfo opUser,
          List<V2TimGroupMemberInfo> memberList) async {
        //某些人被拉入某群（全员能够收到）
        //groupID    群 ID
        //opUser    处理人
        //memberList    被拉进群成员
        _groupListenerData = 'onMemberInvited ${groupID}';
        notifyListeners();
      },
      onMemberKicked: (String groupID, V2TimGroupMemberInfo opUser,
          List<V2TimGroupMemberInfo> memberList) async {
        //某些人被踢出某群（全员能够收到）
        //groupID    群 ID
        //opUser    处理人
        //memberList    被踢成员
        _groupListenerData = 'onMemberKicked ${groupID}';
        notifyListeners();
      },
      onMemberLeave: (String groupID, V2TimGroupMemberInfo member) async {
        //有用户离开群（全员能够收到）
        //groupID    群 ID
        //member    离开的成员
        _groupListenerData = 'onMemberLeave ${groupID}';
        notifyListeners();
      },
      onQuitFromGroup: (String groupID) async {
        //主动退出群组（主要用于多端同步，直播群（AVChatRoom）不支持）
        //groupID    群 ID
        _groupListenerData = 'onQuitFromGroup ${groupID}';
        notifyListeners();
      },
      onReceiveJoinApplication:
          (String groupID, V2TimGroupMemberInfo member, String opReason) async {
        //有新的加群请求（只有群主或管理员会收到）
        //groupID    群 ID
        //member    申请人
        //opReason    申请原因
        _groupListenerData = 'onReceiveJoinApplication ${groupID}';
        notifyListeners();
      },
      onReceiveRESTCustomData: (String groupID, String customData) async {
        //收到 RESTAPI 下发的自定义系统消息
        //groupID    群 ID
        //customData    自定义数据
        _groupListenerData = 'onReceiveRESTCustomData';
        notifyListeners();
      },
      onRevokeAdministrator: (String groupID, V2TimGroupMemberInfo opUser,
          List<V2TimGroupMemberInfo> memberList) async {
        //取消管理员身份
        //groupID    群 ID
        //opUser    处理人
        //memberList    被处理的群成员
        _groupListenerData = 'onRevokeAdministrator ${groupID}';
        notifyListeners();
      },
    );
    //添加群组监听器
    TencentImSDKPlugin.v2TIMManager.addGroupListener(listener: groupListener);
  }

  void removeGroupListener() {
    TencentImSDKPlugin.v2TIMManager
        .removeGroupListener(listener: groupListener);
  }

  List<String> _friendList = [];
  List<String> get friendList => _friendList;

  void getFriendList() async {
    _friendList = [];
    V2TimValueCallback<List<V2TimFriendInfo>> getFriendListRes =
        await TencentImSDKPlugin.v2TIMManager
            .getFriendshipManager()
            .getFriendList();
    if (getFriendListRes.code == 0) {
      getFriendListRes.data?.forEach((element) {
        _friendList.add(element.userID);
      });
    }
    notifyListeners();
  }

  /// Makes `Counter` readable inside the devtools by listing all of its properties
  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties.add(IntProperty('count', count));
    properties.add(StringProperty('msgListenerData', msgListenerData));
    properties.add(
        StringProperty('conversationListenerData', conversationListenerData));
    properties
        .add(StringProperty('friendshipListenerData', friendshipListenerData));
    properties.add(IterableProperty('friendList', friendList));
    properties.add(StringProperty('groupListenerData', groupListenerData));
  }
}
